@@ -4,7 +4,6 @@
 
 use std::collections::BTreeMap;
 use std::collections::VecDeque;
-use std::io::prelude::*;
 
 #[derive(Clone, Default, Debug, PartialEq, Eq)]
 pub(crate) struct TryCmd {
@@ -25,7 +24,8 @@ impl TryCmd {
                 if sequence.steps[0].stdin.is_none() {
                     let stdin_path = path.with_extension("stdin");
                     let stdin = if stdin_path.exists() {
-                        Some(crate::File::read_from(&stdin_path, Some(is_binary))?)
+                        // No `map_text` as we will trust what the user inputted
+                        Some(crate::Data::read_from(&stdin_path, Some(is_binary))?)
                     } else {
                         None
                     };
@@ -35,7 +35,10 @@ impl TryCmd {
                 if sequence.steps[0].expected_stdout.is_none() {
                     let stdout_path = path.with_extension("stdout");
                     let stdout = if stdout_path.exists() {
-                        Some(crate::File::read_from(&stdout_path, Some(is_binary))?)
+                        Some(
+                            crate::Data::read_from(&stdout_path, Some(is_binary))?
+                                .map_text(snapbox::utils::normalize_text),
+                        )
                     } else {
                         None
                     };
@@ -45,7 +48,10 @@ impl TryCmd {
                 if sequence.steps[0].expected_stderr.is_none() {
                     let stderr_path = path.with_extension("stderr");
                     let stderr = if stderr_path.exists() {
-                        Some(crate::File::read_from(&stderr_path, Some(is_binary))?)
+                        Some(
+                            crate::Data::read_from(&stderr_path, Some(is_binary))?
+                                .map_text(snapbox::utils::normalize_text),
+                        )
                     } else {
                         None
                     };
@@ -54,8 +60,9 @@ impl TryCmd {
 
                 sequence
             } else if ext == std::ffi::OsStr::new("trycmd") || ext == std::ffi::OsStr::new("md") {
-                let raw = crate::File::read_from(path, Some(false))?
-                    .into_utf8()
+                let raw = crate::Data::read_from(path, Some(false))?
+                    .map_text(snapbox::utils::normalize_lines)
+                    .into_string()
                     .unwrap();
                 Self::parse_trycmd(&raw)?
             } else {
@@ -95,13 +102,13 @@ impl TryCmd {
             .fs
             .base
             .take()
-            .map(|p| crate::filesystem::resolve_dir(p).map_err(|e| e.to_string()))
+            .map(|p| snapbox::path::resolve_dir(p).map_err(|e| e.to_string()))
             .transpose()?;
         sequence.fs.cwd = sequence
             .fs
             .cwd
             .take()
-            .map(|p| crate::filesystem::resolve_dir(p).map_err(|e| e.to_string()))
+            .map(|p| snapbox::path::resolve_dir(p).map_err(|e| e.to_string()))
             .transpose()?;
 
         Ok(sequence)
@@ -111,8 +118,8 @@ impl TryCmd {
         &self,
         path: &std::path::Path,
         id: Option<&str>,
-        stdout: Option<&crate::File>,
-        stderr: Option<&crate::File>,
+        stdout: Option<&crate::Data>,
+        stderr: Option<&crate::Data>,
     ) -> Result<(), crate::Error> {
         if let Some(ext) = path.extension() {
             if ext == std::ffi::OsStr::new("toml") {
@@ -121,7 +128,7 @@ impl TryCmd {
                 overwrite_toml_output(path, id, stdout, "stdout", "stdout")?;
                 overwrite_toml_output(path, id, stderr, "stderr", "stderr")?;
             } else if ext == std::ffi::OsStr::new("trycmd") || ext == std::ffi::OsStr::new("md") {
-                if stderr.is_some() && stderr != Some(&crate::File::Text("".into())) {
+                if stderr.is_some() && stderr != Some(&crate::Data::new()) {
                     panic!("stderr should have been merged: {:?}", stderr);
                 }
                 if let (Some(id), Some(stdout)) = (id, stdout) {
@@ -140,7 +147,8 @@ impl TryCmd {
                         .to_owned();
                     // Add back trailing newline removed when parsing
                     stdout.push('\n');
-                    let mut raw = crate::File::read_from(path, Some(false))?;
+                    let mut raw = crate::Data::read_from(path, Some(false))?
+                        .map_text(snapbox::utils::normalize_lines);
                     raw.replace_lines(line_nums, &stdout)?;
                     raw.write_to(path)?;
                 }
@@ -157,7 +165,7 @@ impl TryCmd {
     fn parse_trycmd(s: &str) -> Result<Self, crate::Error> {
         let mut steps = Vec::new();
 
-        let mut lines: VecDeque<_> = crate::lines::LinesWithTerminator::new(s)
+        let mut lines: VecDeque<_> = snapbox::utils::LinesWithTerminator::new(s)
             .enumerate()
             .map(|(i, l)| (i + 1, l))
             .collect();
@@ -173,13 +181,13 @@ impl TryCmd {
                         match lang {
                             "trycmd" | "console" => {
                                 if info.any(|i| i == "ignore") {
-                                    debug!("ignore from infostring: {:?}", info);
+                                    snapbox::debug!("ignore from infostring: {:?}", info);
                                 } else {
                                     break;
                                 }
                             }
                             _ => {
-                                debug!("ignore from lang: {:?}", lang);
+                                snapbox::debug!("ignore from lang: {:?}", lang);
                             }
                         }
                     }
@@ -274,7 +282,7 @@ impl TryCmd {
                     stderr_to_stdout: true,
                     expected_status,
                     expected_stdout_source: Some(stdout_start..post_stdout_start),
-                    expected_stdout: Some(crate::File::Text(stdout)),
+                    expected_stdout: Some(crate::Data::text(stdout)),
                     expected_stderr_source: None,
                     expected_stderr: None,
                     binary: false,
@@ -297,7 +305,7 @@ impl TryCmd {
 fn overwrite_toml_output(
     path: &std::path::Path,
     _id: Option<&str>,
-    output: Option<&crate::File>,
+    output: Option<&crate::Data>,
     output_ext: &str,
     output_field: &str,
 ) -> Result<(), crate::Error> {
@@ -305,33 +313,28 @@ fn overwrite_toml_output(
         let output_path = path.with_extension(output_ext);
         if output_path.exists() {
             output.write_to(&output_path)?;
-        } else {
-            match output {
-                crate::File::Binary(_) => {
-                    output.write_to(&output_path)?;
-
-                    let raw = std::fs::read_to_string(path)
-                        .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
-                    let mut doc = raw
-                        .parse::<toml_edit::Document>()
-                        .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
-                    doc[output_field] = toml_edit::Item::None;
-                    std::fs::write(path, doc.to_string())
-                        .map_err(|e| format!("Failed to write {}: {}", path.display(), e))?;
-                }
-                crate::File::Text(output) => {
-                    let raw = std::fs::read_to_string(path)
-                        .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
-                    let mut doc = raw
-                        .parse::<toml_edit::Document>()
-                        .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
-                    if let Some(output_value) = doc.get_mut(output_field) {
-                        *output_value = toml_edit::value(output);
-                    }
-                    std::fs::write(path, doc.to_string())
-                        .map_err(|e| format!("Failed to write {}: {}", path.display(), e))?;
-                }
+        } else if let Some(output) = output.as_str() {
+            let raw = std::fs::read_to_string(path)
+                .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
+            let mut doc = raw
+                .parse::<toml_edit::Document>()
+                .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
+            if let Some(output_value) = doc.get_mut(output_field) {
+                *output_value = toml_edit::value(output);
             }
+            std::fs::write(path, doc.to_string())
+                .map_err(|e| format!("Failed to write {}: {}", path.display(), e))?;
+        } else {
+            output.write_to(&output_path)?;
+
+            let raw = std::fs::read_to_string(path)
+                .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
+            let mut doc = raw
+                .parse::<toml_edit::Document>()
+                .map_err(|e| format!("Failed to read {}: {}", path.display(), e))?;
+            doc[output_field] = toml_edit::Item::None;
+            std::fs::write(path, doc.to_string())
+                .map_err(|e| format!("Failed to write {}: {}", path.display(), e))?;
         }
     }
 
@@ -367,13 +370,13 @@ impl From<OneShot> for TryCmd {
                 bin,
                 args: args.into_vec(),
                 env,
-                stdin: stdin.map(crate::File::Text),
+                stdin: stdin.map(crate::Data::text),
                 stderr_to_stdout,
                 expected_status: status,
                 expected_stdout_source: None,
-                expected_stdout: stdout.map(crate::File::Text),
+                expected_stdout: stdout.map(crate::Data::text),
                 expected_stderr_source: None,
-                expected_stderr: stderr.map(crate::File::Text),
+                expected_stderr: stderr.map(crate::Data::text),
                 binary,
                 timeout,
             }],
@@ -388,13 +391,13 @@ pub(crate) struct Step {
     pub(crate) bin: Option<Bin>,
     pub(crate) args: Vec<String>,
     pub(crate) env: Env,
-    pub(crate) stdin: Option<crate::File>,
+    pub(crate) stdin: Option<crate::Data>,
     pub(crate) stderr_to_stdout: bool,
     pub(crate) expected_status: Option<CommandStatus>,
     pub(crate) expected_stdout_source: Option<std::ops::Range<usize>>,
-    pub(crate) expected_stdout: Option<crate::File>,
+    pub(crate) expected_stdout: Option<crate::Data>,
     pub(crate) expected_stderr_source: Option<std::ops::Range<usize>>,
-    pub(crate) expected_stderr: Option<crate::File>,
+    pub(crate) expected_stderr: Option<crate::Data>,
     pub(crate) binary: bool,
     pub(crate) timeout: Option<std::time::Duration>,
 }
@@ -403,7 +406,7 @@ impl Step {
     pub(crate) fn to_command(
         &self,
         cwd: Option<&std::path::Path>,
-    ) -> Result<std::process::Command, crate::Error> {
+    ) -> Result<snapbox::cmd::Command, crate::Error> {
         let bin = match &self.bin {
             Some(Bin::Path(path)) => Ok(path.clone()),
             Some(Bin::Name(name)) => Err(format!("Unknown bin.name = {}", name).into()),
@@ -414,60 +417,27 @@ impl Step {
             return Err(format!("Bin doesn't exist: {}", bin.display()).into());
         }
 
-        let mut cmd = std::process::Command::new(bin);
-        cmd.args(&self.args);
+        let mut cmd = snapbox::cmd::Command::new(bin).args(&self.args);
         if let Some(cwd) = cwd {
-            cmd.current_dir(cwd);
+            cmd = cmd.current_dir(cwd);
         }
-        self.env.apply(&mut cmd);
+        if let Some(stdin) = &self.stdin {
+            cmd = cmd.stdin(stdin);
+        }
+        if self.stderr_to_stdout {
+            cmd = cmd.stderr_to_stdout();
+        }
+        if let Some(timeout) = self.timeout {
+            cmd = cmd.timeout(timeout)
+        }
+        cmd = self.env.apply(cmd);
 
         Ok(cmd)
     }
 
-    pub(crate) fn to_output(
-        &self,
-        cwd: Option<&std::path::Path>,
-    ) -> Result<std::process::Output, crate::Error> {
-        let mut cmd = self.to_command(cwd)?;
-
-        if self.stderr_to_stdout {
-            cmd.stdin(std::process::Stdio::piped());
-            let (mut reader, writer) = os_pipe::pipe().map_err(|e| e.to_string())?;
-            let writer_clone = writer.try_clone().map_err(|e| e.to_string())?;
-            cmd.stdout(writer);
-            cmd.stderr(writer_clone);
-            let child = cmd.spawn().map_err(|e| e.to_string())?;
-
-            // Avoid a deadlock! This parent process is still holding open pipe
-            // writers (inside the Command object), and we have to close those
-            // before we read. Here we do this by dropping the Command object.
-            drop(cmd);
-
-            let mut output = crate::wait_with_input_output(child, self.stdin(), self.timeout)
-                .map_err(|e| e.to_string())?;
-            assert!(output.stdout.is_empty());
-            assert!(output.stderr.is_empty());
-            reader
-                .read_to_end(&mut output.stdout)
-                .map_err(|e| e.to_string())?;
-            Ok(output)
-        } else {
-            cmd.stdin(std::process::Stdio::piped());
-            cmd.stdout(std::process::Stdio::piped());
-            cmd.stderr(std::process::Stdio::piped());
-            let child = cmd.spawn().map_err(|e| e.to_string())?;
-            crate::wait_with_input_output(child, self.stdin(), self.timeout)
-                .map_err(|e| e.to_string().into())
-        }
-    }
-
     pub(crate) fn expected_status(&self) -> CommandStatus {
         self.expected_status.unwrap_or_default()
     }
-
-    pub(crate) fn stdin(&self) -> Option<&[u8]> {
-        self.stdin.as_ref().map(|f| f.as_bytes())
-    }
 }
 
 /// Top-level data in `cmd.toml` files
@@ -653,14 +623,14 @@ impl Env {
         self.remove.extend(other.remove.iter().cloned());
     }
 
-    pub(crate) fn apply(&self, command: &mut std::process::Command) {
+    pub(crate) fn apply(&self, mut command: snapbox::cmd::Command) -> snapbox::cmd::Command {
         if !self.inherit() {
-            command.env_clear();
+            command = command.env_clear();
         }
         for remove in &self.remove {
-            command.env_remove(&remove);
+            command = command.env_remove(&remove);
         }
-        command.envs(&self.add);
+        command.envs(&self.add)
     }
 
     pub(crate) fn inherit(&self) -> bool {
@@ -761,7 +731,7 @@ mod test {
                 expected_status: Some(CommandStatus::Success),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(4..4),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -788,7 +758,7 @@ $ cmd
                 expected_status: Some(CommandStatus::Success),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(4..4),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -815,7 +785,7 @@ $ cmd arg1 'arg with space'
                 expected_status: Some(CommandStatus::Success),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(5..5),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -850,7 +820,7 @@ $ cmd arg1
                 expected_status: Some(CommandStatus::Success),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(4..4),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -876,7 +846,7 @@ $ KEY1=VALUE1 KEY2='VALUE2 with space' cmd
                 expected_status: Some(CommandStatus::Skipped),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(5..5),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -903,7 +873,7 @@ $ cmd
                 expected_status: Some(CommandStatus::Code(-1)),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(5..5),
-                expected_stdout: Some(crate::File::Text("".into())),
+                expected_stdout: Some(crate::Data::new()),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -930,7 +900,7 @@ $ cmd
                 expected_status: Some(CommandStatus::Success),
                 stderr_to_stdout: true,
                 expected_stdout_source: Some(4..6),
-                expected_stdout: Some(crate::File::Text("Hello World\n".into())),
+                expected_stdout: Some(crate::Data::text("Hello World\n")),
                 expected_stderr: None,
                 ..Default::default()
             }],
@@ -958,7 +928,7 @@ Hello World
                     expected_status: Some(CommandStatus::Code(1)),
                     stderr_to_stdout: true,
                     expected_stdout_source: Some(5..5),
-                    expected_stdout: Some(crate::File::Text("".into())),
+                    expected_stdout: Some(crate::Data::new()),
                     expected_stderr: None,
                     ..Default::default()
                 },
@@ -968,7 +938,7 @@ Hello World
                     expected_status: Some(CommandStatus::Success),
                     stderr_to_stdout: true,
                     expected_stdout_source: Some(6..6),
-                    expected_stdout: Some(crate::File::Text("".into())),
+                    expected_stdout: Some(crate::Data::new()),
                     expected_stderr: None,
                     ..Default::default()
                 },
@@ -998,7 +968,7 @@ $ cmd2
                     expected_status: Some(CommandStatus::Code(1)),
                     stderr_to_stdout: true,
                     expected_stdout_source: Some(5..5),
-                    expected_stdout: Some(crate::File::Text("".into())),
+                    expected_stdout: Some(crate::Data::new()),
                     expected_stderr: None,
                     ..Default::default()
                 },
@@ -1008,7 +978,7 @@ $ cmd2
                     expected_status: Some(CommandStatus::Code(1)),
                     stderr_to_stdout: true,
                     expected_stdout_source: Some(10..10),
-                    expected_stdout: Some(crate::File::Text("".into())),
+                    expected_stdout: Some(crate::Data::new()),
                     expected_stderr: None,
                     ..Default::default()
                 },
@@ -1018,7 +988,7 @@ $ cmd2
                     expected_status: Some(CommandStatus::Code(1)),
                     stderr_to_stdout: true,
                     expected_stdout_source: Some(20..20),
-                    expected_stdout: Some(crate::File::Text("".into())),
+                    expected_stdout: Some(crate::Data::new()),
                     expected_stderr: None,
                     ..Default::default()
                 },
